=== modified file 'lib/lp/code/browser/sourcepackagerecipe.py'
--- lib/lp/code/browser/sourcepackagerecipe.py	2012-01-01 02:58:52 +0000
+++ lib/lp/code/browser/sourcepackagerecipe.py	2012-01-04 07:41:33 +0000
@@ -1,4 +1,4 @@
-# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
+# Copyright 2010-2012 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """SourcePackageRecipe views."""
@@ -350,9 +350,14 @@
         other circumstance which resulted in the build not being completed).
         This allows started but unfinished builds to show up in the view but
         be discarded as more recent builds become available.
+
+        Builds that the user does not have permission to see are excluded.
         """
-        builds = list(recipe.pending_builds)
+        builds = [build for build in recipe.pending_builds
+            if check_permission('launchpad.View', build)]
         for build in recipe.completed_builds:
+            if not check_permission('launchpad.View', build):
+                continue
             builds.append(build)
             if len(builds) >= 5:
                 break
@@ -383,8 +388,10 @@
         """
         initial_values = {'distroseries': self.context.distroseries}
         build = self.context.last_build
-        if build is not None:
-            initial_values['archive'] = build.archive
+        if build:
+            # If the build can't be viewed, the archive can't.
+            if check_permission('launchpad.View', build):
+                initial_values['archive'] = build.archive
         return initial_values
 
     class schema(Interface):

=== modified file 'lib/lp/code/browser/tests/test_sourcepackagerecipe.py'
--- lib/lp/code/browser/tests/test_sourcepackagerecipe.py	2011-12-30 06:14:56 +0000
+++ lib/lp/code/browser/tests/test_sourcepackagerecipe.py	2012-01-04 03:01:58 +0000
@@ -1,4 +1,4 @@
-# Copyright 2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2010-2012 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 # pylint: disable-msg=F0401,E1002
 
@@ -1170,6 +1170,20 @@
               Secret PPA chocolate - 0\+r42 on 2010-04-16 buildlog \(.*\) i386
             Request build\(s\)""", self.getMainText(recipe))
 
+    def test_index_success_with_sprb_into_private_ppa(self):
+        # The index page hides builds into archives the user can't view.
+        recipe = self.makeRecipe()
+        archive = self.factory.makeArchive(private=True)
+        sprb = removeSecurityProxy(
+            self.factory.makeSourcePackageRecipeBuild(
+                recipe=recipe, distroseries=self.squirrel, archive=archive))
+        sprb.status = BuildStatus.FULLYBUILT
+        sprb.date_started = datetime(2010, 04, 16, tzinfo=UTC)
+        sprb.date_finished = datetime(2010, 04, 16, tzinfo=UTC)
+        sprb.log = self.factory.makeLibraryFileAlias()
+        self.assertIn(
+            "This recipe has not been built yet.", self.getMainText(recipe))
+
     def test_index_no_builds(self):
         """A message should be shown when there are no builds."""
         recipe = self.makeRecipe()

